from selenium import webdriver
from selenium.webdriver.chromium.webdriver import ChromiumDriver
from selenium.webdriver.common.by import By
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from dotenv import load_dotenv
from typing import Dict, List
from tempfile import NamedTemporaryFile
from urllib.parse import urlparse, parse_qs
import os
import urllib.parse
import time
from bs4 import BeautifulSoup
from .Browser import Browser
from requests_html import HTMLSession
import pickle
from .utils import parse_profile_page

BASE_URL="https://www.linkedin.com"

load_dotenv()

SCROLL_PAUSE_TIME = 0.5
SCROLL_INCREMENT = 500



# options = webdriver.ChromeOptions()
# options.add_experimental_option(
#         "prefs", {
#             # block image loading
#             "profile.managed_default_content_settings.images": 2,
#         }
#     )

# driver = webdriver.Chrome(
#         options=options
#     )


class LinkedInScraper:
    def __init__(self, username: str, password: str):
        self.browser = Browser(username, password)
        self.driver = self.browser.get_driver()

    def scroll_to_bottom_of_element(driver: ChromiumDriver, element_id: str):
        # Find the element
        scrollable_element =  WebDriverWait(driver, 100).until(
        EC.presence_of_element_located((By.ID, element_id))
    )

        # Get the initial scroll height
        last_height = driver.execute_script("return arguments[0].scrollHeight", scrollable_element)

        while True:
            # Scroll down within the element
            driver.execute_script(f"arguments[0].scrollTop += {SCROLL_INCREMENT};", scrollable_element)

            # Wait for the page to load
            time.sleep(SCROLL_PAUSE_TIME)

            # Calculate new scroll height and compare with last scroll height
            new_height = driver.execute_script("return arguments[0].scrollHeight", scrollable_element)
            print("New Height:", new_height)  # Print the current bottom of the element

            # Check if the bottom has been reached
            if new_height == last_height:
                break
            last_height = new_height
            
    def login():
        pass

    def scrape_profile_page():
        pass

    def scrape_lead_search_page():
        pass

    def get_profiles(self, search_str: str):

        start = time.time()

        browser = self.browser

        driver = browser.get_driver()
    
        # # After successfully logging in
        # pickle.dump(driver.get_cookies(), open("cookies.pkl", "wb"))

        # # At the start of a new session, before navigating to the login page
        # cookies = pickle.load(open("cookies.pkl", "rb"))
        # for cookie in cookies:
        #     driver.add_cookie(cookie)

        search_url = f'{BASE_URL}/sales/search/people'
        params = {
        'query': f'(spellCorrectionEnabled:true,keywords:{search_str})'
        }

        encoded_params = urllib.parse.urlencode(params)

        full_search_url = f"{search_url}?{encoded_params}"


        driver.get(full_search_url)

        WebDriverWait(driver, 100).until(EC.presence_of_element_located((By.ID, 'search-results-container')))
        
        soup = BeautifulSoup(driver.page_source, 'html.parser')

        # Find all 'a' tags with class 'ember-view'
        with open('search.html', 'w') as f:
            f.write(str(soup))

        divs = soup.find_all('div', attrs={'data-scroll-into-view': True})

        # List to hold the extracted URNs
        urns = []

        for div in divs:
            # Extract the value of 'data-scroll-into-view'
            urn = div['data-scroll-into-view']
            # Assuming the format always includes the URN inside parentheses
            urn = urn.split('(')[1].rstrip(')')
            urns.append(urn)

        results = []
        profiles = []


        for urn in urns:
            driver.get(f"{BASE_URL}/sales/lead/{urn}")
            WebDriverWait(driver, 100).until(EC.presence_of_element_located((By.ID, 'content-main')))
            profile_soup = BeautifulSoup(driver.page_source, 'html.parser')
            profile = parse_profile_page(profile_soup)
            with open('profiles.html', 'a') as f:
                f.write(profile_soup.prettify() + "\n\n\n")
            results += profile_soup
            profiles.append(profile)
        

        end = time.time()
        print(f"Time taken: {end - start} seconds")

        return profiles


        # for url in profile_urls:
        #     driver.get(url)
        #     WebDriverWait(driver, 100).until(EC.presence_of_element_located((By.ID, 'profile-content')))

        #     profile_soup = BeautifulSoup(driver.page_source, 'html.parser')
        #     profile_urls_html.append(profile_soup)

        # with open('profiles.html', 'w') as f:
        #     f.write(str(profile_urls_html))



        # scroll_to_bottom_of_element(driver, 'search-results-container')

        # try:
        #     anchors = WebDriverWait(driver, 10).until(
        #         EC.presence_of_all_elements_located((By.XPATH, "//a[@data-control-name='view_lead_panel_via_search_lead_name']"))
        #     )
        #     for anchor in anchors:
        #         print(f"Link: {anchor.get_attribute('href')}, Text: {anchor.text}")
        # except Exception as e:
        #     print(f"An error occurred: {e}")

        # print("ANCHORS", anchors)






        # driver.find_element(By.CSS_SELECTOR, 'global-typeahead__input-label').click() #search bar
        
        # driver.switch_to.active_element.send_keys(args + Keys.RETURN)  # Adding RETURN for demonstration


        input("Press ENTER to continue")



if __name__ == '__main__':
    scraper = LinkedInScraper(os.environ.get('LINKED_IN_LOGIN_EMAIL'), os.environ.get('LINKED_IN_LOGIN_PASSWORD'))
    profiles = scraper.get_profiles('python developer')
    print("PROFILES", profiles)




from dataclasses import dataclass
from typing import List
from Person import Person
from company import Company

@dataclass
class InterestInfo: #TODO: make objects for all of these
    top_voices: List[Person]
    companies: List[Company]
    groups: List[str]
    newsletters: List[str]
    schools: List[str]
    pass


from dataclasses import dataclass
from enum import Enum

@dataclass
class CompanyTypeOptionEnum(Enum):
    EDUCATIONAL = "Educational"
    GOVERNMENT_AGENCY = "Government Agency"
    NON_PROFIT = "Non Profit"
    PARTNERSHIP = "Partnership"
    PRIVATE = "Privately Held"
    PUBLIC = "Public Company"
    SELF_EMPLOYED = "Self Employed"
    SELF_OWNED = "Self Owned"



@dataclass
class CompanySizeOptionEnum(Enum):
    SMALL = "1-10 employees"
    MEDIUM = "11-50 employees"
    LARGE = "51-200 employees"
    VERY_LARGE = "201-500 employees"
    HUGE = "501-1000 employees"
    VERY_HUGE = "1001-5000 employees"
    MASSIVE = "5001-10000 employees"
    VERY_MASSIVE = "10001+ employees"

@dataclass
class Company:
    name: str
    linkedin_url: str
    tagline: str
    description: str
    industry: str
    company_size: CompanySizeOptionEnum
    company_type: CompanyTypeOptionEnum
    phone: str
    year_founded: int
    location: str #not sure what to do with this
    website: str #has to be scraped somehow



from bs4 import BeautifulSoup
import re
from models import Person, BasicInfo, PersonalLocation, Experience, Post
from types import List


def parse_experience_entry(experience_entry: BeautifulSoup) -> Experience:

    if (job_title_tag:= experience_entry.find('h3', {'data-anonymize': 'job-title'})) is not None:
        job_title = job_title_tag.get_text(strip=True)
    else:
        job_title = ""

    if (company_name_tag := experience_entry.find('span', {'data-anonymize': 'company-name'})) is not None:
        company_name = company_name_tag.get_text(strip=True)
    else:
        company_name = ""
    
    #TODO: Find a way to get company_link

    if (person_blurb_tag := experience_entry.find('div', {'data-anonymize': 'person-blurb'})) is not None:
        person_blurb = person_blurb_tag.get_text(strip=True)
    else:
        person_blurb = ""

    # Extracting time period
    if (time_period_tag := experience_entry.find('span', {'data-anonymize': 'time-period'})) is not None:
        time_period = time_period_tag.get_text(strip=True)
    else:
        time_period = ""

    if (location_tag := experience_entry.find('span', {'data-anonymize': 'location'})) is not None:
        location = location_tag.get_text(strip=True)
    else:
        location = ""

    if (job_description_tag := experience_entry.find('div', {'data-anonymize': 'job-description'})) is not None:
        job_description = job_description_tag.get_text(strip=True)
    else:
        job_description = ""

    # Append to experiences list
    experience = Experience(job_title, company_name, time_period, location, person_blurb, job_description)

    return experience


def parse_profile_page(soup: BeautifulSoup) -> Person:

    # Extracting specific information
    # Name and additional info (e.g., professional titles)
    if (name_tag := soup.find('p', {'data-anonymize': 'person-name'})) is not None:
        name = name_tag.get_text(strip=True)
    else: 
        name = ""

    if (headline_tag := soup.find('p', {'data-anonymize': 'headline'})) is not None:
        headline = headline_tag.get_text(strip=True)
    else:
        headline = ""

    experience_lst = soup.find("div", {"id": "experience-section"}).find('ul')
    experiences: List[Experience] = []

    for li in experience_lst.find_all('li'):
        experiences.append(parse_experience_entry(li))
    
    # pattern = re.compile(r'.*\b_current-role-item_\b.*')  # ensures the class name contains '_current-role-item_'
    # role_items = soup.find_all('p', class_=pattern)

    # Location
    if (location_tag := soup.select_one('EzlYHbcfhKpSiteJvPPqZaMLmlAeyUkcBi')) is not None:
        location_text = location_tag.get_text(strip=True)
        city, country = location_text.split(', ')[0], location_text.split(', ')[-1]
    else:
        location_text = ""
        city, country = "", ""
    

    # Contact Information
    # This might be more complex depending on how it's structured in the HTML. This is an example:
    contact_info = "Not directly available"

    # Interest information
    # This also needs to be extracted based on specific identifiers or classes in the HTML
    interest_info = "Needs specific identifiers"

    # Output
    print(f"Name: {name}")
    print(f"City: {city}, Country: {country}")
    print("Experiences:", experiences)

    basic_info = BasicInfo(name, headline)
    personal_location = PersonalLocation(city, "", country)

    profile = Person("", basic_info, "", "", personal_location, "", "", [], experiences)

    return profile



# Use site:linkedin.com

from selenium import webdriver
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.by import By
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from dotenv import load_dotenv
from typing import Dict, List
from bs4 import BeautifulSoup
from tempfile import NamedTemporaryFile
from urllib.parse import urlparse, parse_qs



load_dotenv()

options = webdriver.ChromeOptions()
options.add_experimental_option(
        "prefs", {
            # block image loading
            "profile.managed_default_content_settings.images": 2,
        }
    )
driver = webdriver.Chrome(
        options=options
    )

def construct_main_query(options: Dict[str, List[str]]):
    query = "site:linkedin.com/in/"

    keywords = options.get("keywords")

    for keyword in keywords:
        query += f" AND {keyword}"

    return query
    

def main():
    driver.get('https://www.google.com/search?q=site:linkedin.com/in/ AND "python developer" AND "London"')
    input('Press ENTER to continue')

    linkedin_links = []
    
    page_source = driver.page_source
    
    soup = BeautifulSoup(page_source, 'html.parser')
    links = soup.find_all('a')
    for link in links:
        href = link.get('href', '')
        parsed_url = urlparse(href)
        
        # Check if the domain is 'linkedin.com' and the path starts with '/in/'
        if parsed_url.netloc == 'www.linkedin.com' and parsed_url.path.startswith('/in/'):
            # Ensure no query or fragment is specifying 'linkedin.com/in/' misleadingly
            if 'linkedin.com/in/' not in parse_qs(parsed_url.query).values() and 'linkedin.com/in/' not in parsed_url.fragment:
                linkedin_links.append(href)
    
    print("LINKEDIN LINKS", linkedin_links)

    for link in linkedin_links:
        driver.get(link)
        linkedin_page = BeautifulSoup(driver.page_source, 'html.parser')
        with open('linkedin.txt', 'a') as f:
            f.write(linkedin_page.prettify() + "\n\n\n")


if __name__ == "__main__":
    main()



from selenium.webdriver.chrome.options import Options
from selenium import webdriver
import traceback
import time
import pickle
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
import os

class Browser:
    def __init__(self,username,password):
        start = time.time()
        chrome_options = Options(
        )
        chrome_options.add_experimental_option(
        "prefs", {
            # block image loading
            "profile.managed_default_content_settings.images": 2,
        }
    )
        
        prefs = {
            "download_restrictions": 3,
        }
        chrome_options.add_experimental_option(
            "prefs", prefs
        )
        chrome_options.page_load_strategy = 'none'  # Load pages without waiting for all resources
        chrome_options.add_argument("--disable-javascript")
        driver = webdriver.Chrome(options = chrome_options)
        
        self.driver=driver          
        self.username=username
        self.password=password
        self.login()
        end = time.time()
        print(f"Time taken to login: {end - start} seconds")

    def get_driver(self):
        return self.driver
        
    def login(self):
        self.driver.get('https://www.linkedin.com/sales/login')

        try:
            print("trying to load cookie if available")
            self.loadcookie()
            return
        except:
            print("some problem with cookie or its not available")
            traceback.print_exc()
            

        WebDriverWait(self.driver, 10).until(EC.frame_to_be_available_and_switch_to_it((By.CLASS_NAME, "authentication-iframe")))

        username = self.driver.find_element(By.ID, 'username')
        username.send_keys(os.environ.get('LINKED_IN_LOGIN_EMAIL'))

        password = self.driver.find_element(By.ID, 'password')
        password.send_keys(os.environ.get('LINKED_IN_LOGIN_PASSWORD'))
        password.send_keys(Keys.RETURN)


        self.savecookies()

    def loadcookie(self):
        print("loading cookie")
        cookies = pickle.load(open("cookies.pkl", "rb"))
        print(cookies)
        self.driver.add_cookie(cookies)
        print('loaded cookie')


    def savecookies(self):
        print("saving cookie")
        time.sleep(10)
        cookies=self.driver.get_cookies()
        for cookie in cookies:
            if(cookie['name']=='li_at'):
                cookie['domain']='.linkedin.com'
                x={
                'name': 'li_at',
                'value': cookie['value'],
                'domain': '.linkedin.com'
                }
                break
        pickle.dump(x , open("cookies.pkl","wb"))
        print('cookies saved')


from dataclasses import dataclass
from Experience import Experience
from typing import List
from Education import Education
from Post import Post
from InterestInfo import InterestInfo

@dataclass
class PersonalLocation:
    city: str
    postal_code: str
    country: str

@dataclass
class Name:
    first: str
    last: str
    addiional: str

@dataclass
class BasicInfo:
    name: Name
    headline: str
    # current_position: Experience #Not really sure what this dropdown is pulling from


@dataclass
class Person:
    about: str
    basic: BasicInfo
    industry: str
    education: str
    location: PersonalLocation
    contact_info: str
    interest_info: InterestInfo
    posts: List[Post]
    experiences: List[Experience]




from dataclasses import dataclass
from models.person.common import Position


@dataclass
class Education(Position):
    school: str
    degree: str
    field_of_study: str
    grade: str
    activities_and_societies: str



from dataclasses import dataclass
from typing import List

@dataclass
class Post:
    post_id: int
    links: List[str]
    content: str
    date: str
    interactions: int #TODO: could be more granular about the type of interaction here
    comments: int
    shares: int
    hashtags: List[str]
    pass


from setuptools import setup, find_packages

setup(
    name='cometai_core',
    version='0.0.7',
    packages=find_packages(),
    description='A simple example package',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='Ethan Hou',
    author_email='ethanfhou10@gmail.com',
    url='https://github.com/cometai/core',
    license='MIT',
    install_requires=[
       'appdirs',
        'attrs',
        'beautifulsoup4',
        'bs4',
        'certifi',
        'charset-normalizer',
        'cssselect',
        'exceptiongroup',
        'fake-useragent',
        'h11',
        'idna',
        'importlib_metadata',
        'importlib_resources',
        'lxml',
        'lxml_html_clean',
        'outcome',
        'parse',
        'pyee',
        'pyppeteer',
        'pyquery',
        'PySocks',
        'python-dotenv',
        'requests',
        'requests-html',
        'selenium',
        'sniffio',
        'sortedcontainers',
        'soupsieve',
        'trio',
        'trio-websocket',
        'typing_extensions',
        'urllib3',
        'w3lib',
        'websockets',
        'wsproto',
        'zipper'
    ],
)



from selenium import webdriver
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.by import By
from selenium.webdriver.chrome.options import Options
import time
from dotenv import load_dotenv
import os

load_dotenv()

options = webdriver.ChromeOptions()
options.add_experimental_option(
        "prefs", {
            # block image loading
            "profile.managed_default_content_settings.images": 2,
        }
    )
driver = webdriver.Chrome(
        options=options
    )



from dataclasses import dataclass
from enum import Enum
from models.person.common import Position
from company import Company

@dataclass
class LocationTypeEnum(Enum):
    ON_SITE = "On-site"
    HYBRID = "Hybrid"
    REMOTE = "Remote"

@dataclass
class Location:
    location: str
    location_type: LocationTypeEnum

@dataclass
class EmploymentTypeEnum(Enum): 
    FULL_TIME = "Full-time"
    PART_TIME = "Part-time"
    SELF_EMPLOYED = "Self-employed"
    CONTRACT = "Contract"
    INTERNSHIP = "Internship"
    SEASONAL = "Seasonal"
    FREELANCE = "Freelance"
    APPRENTICESHIP = "Apprenticeship"

@dataclass
class Experience(Position):
    title: str
    employment_type: EmploymentTypeEnum
    company: Company
    location: Location
    currently_working: bool
    start_date: str
    end_date: str
    description: str
    profile_headline: str
    pass
